@@ -47,7 +47,7 @@ from mypy import message_registry, errorcodes as codes
 from mypy.errors import Errors
 from mypy.fastparse import (
     TypeConverter, parse_type_comment, bytes_to_human_readable_repr, parse_type_ignore_tag,
-    TYPE_IGNORE_PATTERN
+    TYPE_IGNORE_PATTERN, INVALID_TYPE_IGNORE
 )
 from mypy.options import Options
 from mypy.reachability import mark_block_unreachable
@@ -342,8 +342,13 @@ class ASTConverter:
         return id
 
     def visit_Module(self, mod: ast27.Module) -> MypyFile:
-        self.type_ignores = {ti.lineno: parse_type_ignore_tag(ti.tag)  # type: ignore[attr-defined]
-                             for ti in mod.type_ignores}
+        self.type_ignores = {}
+        for ti in mod.type_ignores:
+            parsed = parse_type_ignore_tag(ti.tag)  # type: ignore[attr-defined]
+            if parsed is not None:
+                self.type_ignores[ti.lineno] = parsed
+            else:
+                self.fail(INVALID_TYPE_IGNORE, ti.lineno, -1)
         body = self.fix_function_overloads(self.translate_stmt_list(mod.body))
         return MypyFile(body,
                         self.imports,
@@ -553,7 +558,10 @@ class ASTConverter:
                 if extra_ignore:
                     tag = cast(Any, extra_ignore).group(1)  # type: Optional[str]
                     ignored = parse_type_ignore_tag(tag)
-                    self.type_ignores[converter.line] = ignored
+                    if ignored is None:
+                        self.fail(INVALID_TYPE_IGNORE, converter.line, -1)
+                    else:
+                        self.type_ignores[converter.line] = ignored
                 return typ
         return None
 
