@@ -20,13 +20,30 @@ from mypyc.sametype import is_same_type
 
 
 class HeaderDeclaration:
+    """A representation of a declaration in C.
+
+    This is used to generate declarations in header files and
+    (optionally) definitions in source files.
+
+    Attributes:
+      decl: C source code for the declaration.
+      defn: Optionally, C source code for a definition.
+      dependencies: The names of any objects that must be declared prior.
+      is_type: Whether the declaration is of a C type. (C types will be declared in
+               external header files and not marked 'extern'.)
+    """
+
     def __init__(self,
-                 dependencies: Set[str], decl: List[str], defn: Optional[List[str]],
-                 needs_extern: bool = False) -> None:
-        self.dependencies = dependencies
-        self.decl = decl
+                 decl: Union[str, List[str]],
+                 defn: Optional[List[str]] = None,
+                 *,
+                 dependencies: Optional[Set[str]] = None,
+                 is_type: bool = False
+                 ) -> None:
+        self.decl = [decl] if isinstance(decl, str) else decl
         self.defn = defn
-        self.needs_extern = needs_extern
+        self.dependencies = dependencies or set()
+        self.is_type = is_type
 
 
 class EmitterContext:
@@ -152,7 +169,11 @@ class Emitter:
         return '{}{}'.format(NATIVE_PREFIX, fn.cname(self.names))
 
     def tuple_c_declaration(self, rtuple: RTuple) -> List[str]:
-        result = ['struct {} {{'.format(rtuple.struct_name)]
+        result = [
+            '#ifndef MYPYC_DECLARED_{}'.format(rtuple.struct_name),
+            '#define MYPYC_DECLARED_{}'.format(rtuple.struct_name),
+            'typedef struct {} {{'.format(rtuple.struct_name),
+        ]
         if len(rtuple.types) == 0:  # empty tuple
             # Empty tuples contain a flag so that they can still indicate
             # error values.
@@ -162,7 +183,11 @@ class Emitter:
             for typ in rtuple.types:
                 result.append('{}f{};'.format(self.ctype_spaced(typ), i))
                 i += 1
-        result.append('};')
+        result.append('}} {};'.format(rtuple.struct_name))
+        values = self.tuple_undefined_value_helper(rtuple)
+        result.append('static {} {} = {{ {} }};'.format(
+            self.ctype(rtuple), self.tuple_undefined_value(rtuple), ''.join(values)))
+        result.append('#endif')
         result.append('')
 
         return result
@@ -183,17 +208,7 @@ class Emitter:
                 tuple_expr_in_c, compare, c_type_compare_val(item_type))
 
     def tuple_undefined_value(self, rtuple: RTuple) -> str:
-        context = self.context
-        id = rtuple.unique_id
-        name = 'tuple_undefined_' + id
-        if name not in context.declarations:
-            values = self.tuple_undefined_value_helper(rtuple)
-            var = 'struct {} {}'.format(rtuple.struct_name, name)
-            decl = '{};'.format(var)
-            init = '{} = {{ {} }};'.format(var, ''.join(values))
-            context.declarations[name] = HeaderDeclaration(
-                set([rtuple.struct_name]), [decl], [init])
-        return name
+        return 'tuple_undefined_' + rtuple.unique_id
 
     def tuple_undefined_value_helper(self, rtuple: RTuple) -> List[str]:
         res = []
@@ -222,9 +237,9 @@ class Emitter:
                     dependencies.add(typ.struct_name)
 
             self.context.declarations[tuple_type.struct_name] = HeaderDeclaration(
-                dependencies,
                 self.tuple_c_declaration(tuple_type),
-                None,
+                dependencies=dependencies,
+                is_type=True,
             )
 
     def emit_inc_ref(self, dest: str, rtype: RType) -> None:
@@ -622,7 +637,7 @@ class Emitter:
                     inner_name = self.temp_name()
                     self.emit_box('{}.f{}'.format(src, i), inner_name, typ.types[i],
                                   declare_dest=True)
-                    self.emit_line('PyTuple_SET_ITEM({}, {}, {});'.format(dest, i, inner_name, i))
+                    self.emit_line('PyTuple_SET_ITEM({}, {}, {});'.format(dest, i, inner_name))
         else:
             assert not typ.is_unboxed
             # Type is boxed -- trivially just assign.
