@@ -4,8 +4,8 @@
 from typing import Optional, List, Tuple, Dict, Callable, Mapping, Set
 from collections import OrderedDict
 
-from mypyc.common import NATIVE_PREFIX, PREFIX, REG_PREFIX
-from mypyc.emit import Emitter
+from mypyc.common import PREFIX, NATIVE_PREFIX, REG_PREFIX
+from mypyc.emit import Emitter, HeaderDeclaration
 from mypyc.emitfunc import native_function_header, native_getter_name, native_setter_name
 from mypyc.emitwrapper import (
     generate_dunder_wrapper, generate_hash_wrapper, generate_richcompare_wrapper,
@@ -69,7 +69,8 @@ SIDE_TABLES = [
 
 def generate_slots(cl: ClassIR, table: SlotTable, emitter: Emitter) -> Dict[str, str]:
     fields = OrderedDict()  # type: Dict[str, str]
-    for name, (slot, generator) in table.items():
+    # Sort for determinism on Python 3.5
+    for name, (slot, generator) in sorted(table.items()):
         method = cl.get_method(name)
         if method:
             fields[slot] = generator(cl, method, emitter)
@@ -77,16 +78,20 @@ def generate_slots(cl: ClassIR, table: SlotTable, emitter: Emitter) -> Dict[str,
     return fields
 
 
-def generate_class_type_decl(cl: ClassIR, c_emitter: Emitter, emitter: Emitter) -> None:
-    c_emitter.emit_line('PyTypeObject *{};'.format(emitter.type_struct_name(cl)))
-    emitter.emit_line('extern PyTypeObject *{};'.format(emitter.type_struct_name(cl)))
-    emitter.emit_line()
-    generate_object_struct(cl, emitter)
-    emitter.emit_line()
+def generate_class_type_decl(cl: ClassIR, c_emitter: Emitter,
+                             external_emitter: Emitter,
+                             emitter: Emitter) -> None:
+    context = c_emitter.context
+    name = emitter.type_struct_name(cl)
+    context.declarations[name] = HeaderDeclaration(
+        'PyTypeObject *{};'.format(emitter.type_struct_name(cl)))
+
+    generate_object_struct(cl, external_emitter)
     declare_native_getters_and_setters(cl, emitter)
     generate_full = not cl.is_trait and not cl.builtin_base
     if generate_full:
-        emitter.emit_line('{};'.format(native_function_header(cl.ctor, emitter)))
+        context.declarations[emitter.native_function_name(cl.ctor)] = HeaderDeclaration(
+            '{};'.format(native_function_header(cl.ctor, emitter)))
 
 
 def generate_class(cl: ClassIR, module: str, emitter: Emitter) -> None:
@@ -197,7 +202,7 @@ def generate_class(cl: ClassIR, module: str, emitter: Emitter) -> None:
         generate_dealloc_for_class(cl, dealloc_name, clear_name, emitter)
         emit_line()
         generate_native_getters_and_setters(cl, emitter)
-        vtable_name = generate_vtables(cl, vtable_name, emitter)
+        vtable_name = generate_vtables(cl, vtable_setup_name, vtable_name, emitter)
         emit_line()
     if needs_getseters:
         generate_getseter_declarations(cl, emitter)
@@ -221,7 +226,6 @@ def generate_class(cl: ClassIR, module: str, emitter: Emitter) -> None:
         t=emitter.type_struct_name(cl)))
 
     emitter.emit_line()
-    generate_trait_vtable_setup(cl, vtable_setup_name, vtable_name, emitter)
     if generate_full:
         generate_setup_for_class(cl, setup_name, defaults_fn, vtable_name, emitter)
         emitter.emit_line()
@@ -241,30 +245,42 @@ def setter_name(cl: ClassIR, attribute: str, names: NameGenerator) -> str:
 
 
 def generate_object_struct(cl: ClassIR, emitter: Emitter) -> None:
-    emitter.emit_lines('typedef struct {',
-                       'PyObject_HEAD',
-                       'CPyVTableItem *vtable;')
     seen_attrs = set()  # type: Set[Tuple[str, RType]]
+    lines = []  # type: List[str]
+    lines += ['typedef struct {',
+              'PyObject_HEAD',
+              'CPyVTableItem *vtable;']
     for base in reversed(cl.base_mro):
         if not base.is_trait:
             for attr, rtype in base.attributes.items():
                 if (attr, rtype) not in seen_attrs:
-                    emitter.emit_line('{}{};'.format(emitter.ctype_spaced(rtype),
-                                                     emitter.attr(attr)))
+                    lines.append('{}{};'.format(emitter.ctype_spaced(rtype),
+                                                emitter.attr(attr)))
                     seen_attrs.add((attr, rtype))
-    emitter.emit_line('}} {};'.format(cl.struct_name(emitter.names)))
+    lines.append('}} {};'.format(cl.struct_name(emitter.names)))
+    lines.append('')
+    emitter.context.declarations[cl.struct_name(emitter.names)] = HeaderDeclaration(
+        lines,
+        is_type=True
+    )
 
 
 def declare_native_getters_and_setters(cl: ClassIR,
                                        emitter: Emitter) -> None:
+    decls = emitter.context.declarations
     for attr, rtype in cl.attributes.items():
-        emitter.emit_line('{}{}({} *self);'.format(emitter.ctype_spaced(rtype),
-                                                   native_getter_name(cl, attr, emitter.names),
-                                                   cl.struct_name(emitter.names)))
-        emitter.emit_line(
+        getter_name = native_getter_name(cl, attr, emitter.names)
+        setter_name = native_setter_name(cl, attr, emitter.names)
+        decls[getter_name] = HeaderDeclaration(
+            '{}{}({} *self);'.format(emitter.ctype_spaced(rtype),
+                                     getter_name,
+                                     cl.struct_name(emitter.names))
+        )
+        decls[setter_name] = HeaderDeclaration(
             'bool {}({} *self, {}value);'.format(native_setter_name(cl, attr, emitter.names),
                                                  cl.struct_name(emitter.names),
-                                                 emitter.ctype_spaced(rtype)))
+                                                 emitter.ctype_spaced(rtype))
+        )
 
 
 def generate_native_getters_and_setters(cl: ClassIR,
@@ -306,24 +322,51 @@ def generate_native_getters_and_setters(cl: ClassIR,
 
 
 def generate_vtables(base: ClassIR,
+                     vtable_setup_name: str,
                      vtable_name: str,
                      emitter: Emitter) -> str:
-    """Emit the vtables for a class.
+    """Emit the vtables and vtable setup functions for a class.
 
     This includes both the primary vtable and any trait implementation vtables.
 
+    To account for both dynamic loading and dynamic class creation,
+    vtables are populated dynamically at class creation time, so we
+    emit empty array definitions to store the vtables and a function to
+    populate them.
+
     Returns the expression to use to refer to the vtable, which might be
-    different than the name, if there are trait vtables."""
+    different than the name, if there are trait vtables.
+    """
+
+    def trait_vtable_name(trait: ClassIR) -> str:
+        return '{}_{}_trait_vtable'.format(
+            base.name_prefix(emitter.names), trait.name_prefix(emitter.names))
+
+    # Emit array definitions with enough space for all the entries
+    emitter.emit_line('static CPyVTableItem {}[{}];'.format(
+        vtable_name,
+        max(1, len(base.vtable_entries) + 2 * len(base.trait_vtables))))
+    for trait, vtable in base.trait_vtables.items():
+        emitter.emit_line('static CPyVTableItem {}[{}];'.format(
+            trait_vtable_name(trait),
+            max(1, len(vtable))))
+
+    # Emit vtable setup function
+    emitter.emit_line('static bool')
+    emitter.emit_line('{}{}(void)'.format(NATIVE_PREFIX, vtable_setup_name))
+    emitter.emit_line('{')
 
     subtables = []
     for trait, vtable in base.trait_vtables.items():
-        name = '{}_{}_trait_vtable'.format(
-            base.name_prefix(emitter.names), trait.name_prefix(emitter.names))
+        name = trait_vtable_name(trait)
         generate_vtable(vtable, name, emitter, [])
         subtables.append((trait, name))
 
     generate_vtable(base.vtable_entries, vtable_name, emitter, subtables)
 
+    emitter.emit_line('return 1;')
+    emitter.emit_line('}')
+
     return vtable_name if not subtables else "{} + {}".format(vtable_name, len(subtables) * 2)
 
 
@@ -331,14 +374,11 @@ def generate_vtable(entries: VTableEntries,
                     vtable_name: str,
                     emitter: Emitter,
                     subtables: List[Tuple[ClassIR, str]]) -> None:
-    emitter.emit_line('static CPyVTableItem {}[] = {{'.format(vtable_name))
+    emitter.emit_line('CPyVTableItem {}_scratch[] = {{'.format(vtable_name))
     if subtables:
         emitter.emit_line('/* Array of trait vtables */')
         for trait, table in subtables:
-            # N.B: C only lets us store constant values. We do a nasty hack of
-            # storing a pointer to the location, which we will then dynamically
-            # patch up on module load in CPy_FixupTraitVtable.
-            emitter.emit_line('(CPyVTableItem)&{}, (CPyVTableItem){},'.format(
+            emitter.emit_line('(CPyVTableItem){}, (CPyVTableItem){},'.format(
                 emitter.type_struct_name(trait), table))
         emitter.emit_line('/* Start of real vtable */')
 
@@ -354,24 +394,7 @@ def generate_vtable(entries: VTableEntries,
     if not entries:
         emitter.emit_line('NULL')
     emitter.emit_line('};')
-
-
-def generate_trait_vtable_setup(cl: ClassIR,
-                                vtable_setup_name: str,
-                                vtable_name: str,
-                                emitter: Emitter) -> None:
-    """Generate a native function that fixes up the trait vtables of a class.
-
-    This needs to be called before a class is used.
-    """
-    emitter.emit_line('static bool')
-    emitter.emit_line('{}{}(void)'.format(NATIVE_PREFIX, vtable_setup_name))
-    emitter.emit_line('{')
-    if cl.trait_vtables and not cl.is_trait:
-        emitter.emit_lines('CPy_FixupTraitVtable({}_vtable, {});'.format(
-            cl.name_prefix(emitter.names), len(cl.trait_vtables)))
-    emitter.emit_line('return 1;')
-    emitter.emit_line('}')
+    emitter.emit_line('memcpy({name}, {name}_scratch, sizeof({name}));'.format(name=vtable_name))
 
 
 def generate_setup_for_class(cl: ClassIR,
@@ -420,14 +443,27 @@ def generate_constructor_for_class(cl: ClassIR,
     emitter.emit_line('PyObject *self = {}();'.format(setup_name))
     emitter.emit_line('if (self == NULL)')
     emitter.emit_line('    return NULL;')
+    args = ', '.join(['self'] + [REG_PREFIX + arg.name for arg in fn.sig.args])
     if init_fn is not None:
-        args = ', '.join(['self'] + [REG_PREFIX + arg.name for arg in fn.sig.args])
         emitter.emit_line('char res = {}{}({});'.format(
             NATIVE_PREFIX, init_fn.cname(emitter.names), args))
         emitter.emit_line('if (res == 2) {')
         emitter.emit_line('Py_DECREF(self);')
         emitter.emit_line('return NULL;')
         emitter.emit_line('}')
+
+    # If there is a nontrivial ctor that we didn't define, invoke it via tp_init
+    elif len(fn.sig.args) > 1:
+        emitter.emit_line(
+            'int res = {}->tp_init({});'.format(
+                emitter.type_struct_name(cl),
+                args))
+
+        emitter.emit_line('if (res < 0) {')
+        emitter.emit_line('Py_DECREF(self);')
+        emitter.emit_line('return NULL;')
+        emitter.emit_line('}')
+
     emitter.emit_line('return self;')
     emitter.emit_line('}')
 
